<?php
/**
 * This is the entrypoint for the locrMAPS for WooCommerce WordPress plugin.
 *
 * @package locrMAPS for WooCommerce
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\ValueObjects;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the style.
 *
 * @property-read Dpi      $Dpi    The DPI.
 * @property-read Height   $Height The height.
 * @property-read string   $Name   The name.
 * @property-read SizeUnit $Unit   The unit.
 * @property-read Width    $Width  The width.
 */
final class Size {
	/**
	 * The constructor.
	 *
	 * @param Width    $width The width.
	 * @param Height   $height The height.
	 * @param SizeUnit $unit The unit.
	 * @param Dpi      $dpi The DPI.
	 * @param string   $name The name.
	 */
	public function __construct(
		private Width $width,
		private Height $height,
		private SizeUnit $unit = SizeUnit::CM,
		private Dpi $dpi = new Dpi(),
		private string $name = ''
	) {
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @return mixed
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'Dpi' => $this->dpi,
			'Height' => $this->height,
			'Name' => $this->name,
			'Unit' => $this->unit,
			'Width' => $this->width,
			default => throw new \InvalidArgumentException("Property $name does not exist"), // phpcs:ignore
		};
	}

	/**
	 * Build the display name.
	 */
	public function build_display_name(): string {
		if ( ! empty( $this->name ) ) {
			return $this->name;
		}

		return sprintf(
			'%sx%s%s',
			$this->width->Value,
			$this->height->Value,
			$this->unit->value
		);
	}

	/**
	 * Build the Size from a shortcode.
	 *
	 * @param string $shortcode The shortcode.
	 * @throws \InvalidArgumentException If the shortcode is invalid.
	 */
	public static function build_from_shortcode( string $shortcode ): self {
		$shortcode_regex = get_shortcode_regex( array( 'size' ) );
		if ( ! preg_match( "/$shortcode_regex/", $shortcode, $matches ) ) {
			throw new \InvalidArgumentException( 'Invalid shortcode name' );
		}
		$parsed_atts = shortcode_parse_atts( trim( $matches[3] ) );
		if ( empty( $parsed_atts ) ) {
			throw new \InvalidArgumentException( 'No shortcode attributes found' );
		}
		if ( ! isset( $parsed_atts['width'] ) ) {
			throw new \InvalidArgumentException( 'No width attribute in shortcode found' );
		}
		if ( ! is_numeric( $parsed_atts['width'] ) ) {
			throw new \InvalidArgumentException( 'The width attribute must be a number' );
		}
		if ( ! isset( $parsed_atts['height'] ) ) {
			throw new \InvalidArgumentException( 'No height attribute in shortcode found' );
		}
		if ( ! is_numeric( $parsed_atts['height'] ) ) {
			throw new \InvalidArgumentException( 'The height attribute must be a number' );
		}

		$unit = SizeUnit::CM;
		if ( isset( $parsed_atts['unit'] ) && is_string( $parsed_atts['unit'] ) ) {
			$unit = SizeUnit::tryFrom( $parsed_atts['unit'] ) ?? throw new \InvalidArgumentException( 'The unit attribute is invalid' );
		}

		$dpi = new Dpi();
		if ( isset( $parsed_atts['dpi'] ) ) {
			if ( ! is_numeric( $parsed_atts['dpi'] ) ) {
				throw new \InvalidArgumentException( 'The dpi attribute must be a number' );
			}
			$dpi = new Dpi( intval( $parsed_atts['dpi'] ) );
		}

		$name = '';
		if ( isset( $parsed_atts['name'] ) && is_string( $parsed_atts['name'] ) ) {
			$name = trim( $parsed_atts['name'] );
		}

		return new self(
			width: new Width( floatval( $parsed_atts['width'] ) ),
			height: new Height( floatval( $parsed_atts['height'] ) ),
			unit: $unit,
			dpi: $dpi,
			name: $name
		);
	}

	/**
	 * Build the option value.
	 */
	public function build_option_value(): string {
		return sprintf(
			'%sx%s%s@%sdpi',
			$this->width->Value,
			$this->height->Value,
			$this->unit->value,
			$this->dpi->Value
		);
	}
}
